@@ -11,7 +11,7 @@
 #
 ##################################################################
 #
-# $Id: Parse.pm 1595 2006-09-06 00:00:30Z mbr $
+# $Id: Parse.pm 1978 2007-02-28 02:13:20Z mbr $
 #
 
 package IPTables::Parse;
@@ -84,8 +84,14 @@ sub chain_rules() {
     my $chain  = shift || croak '[*] Specify a chain, e.g. "OUTPUT"';
     my $file   = shift || '';
     my $iptables  = $self->{'_iptables'};
+
+    my $found_chain  = 0;
     my @ipt_lines;
 
+    ### array of hash refs
+    my @chain = ();
+    my @global_accept_state = ();
+
     if ($file) {
         ### read the iptables rules out of $file instead of executing
         ### the iptables command.
@@ -101,11 +107,6 @@ sub chain_rules() {
         };
     }
 
-    my $found_chain = 0;
-
-    ### array of hash refs
-    my @chain = ();
-
     ### determine the output style (e.g. "-nL -v" or just plain "-nL"; if the
     ### policy data came from a file then -v might not have been used)
     my $ipt_verbose = 0;
@@ -148,7 +149,8 @@ sub chain_rules() {
             'd_port'   => '',
             'dport'    => '',
             'extended' => '',
-            'raw'      => ''  ### only used if regex doesn't match
+            'state'    => '',
+            'raw'      => $line
         );
 
         if ($ipt_verbose) {
@@ -167,22 +169,52 @@ sub chain_rules() {
                 $rule{'src'}      = $7;
                 $rule{'dst'}      = $8;
                 $rule{'extended'} = $9;
-                if ($rule{'extended'}
-                        and ($rule{'protocol'} eq 'tcp'
-                        or $rule{'protocol'} eq 'udp')) {
-                    my $s_port  = '0:0';  ### any to any
-                    my $d_port  = '0:0';
-                    if ($rule{'extended'} =~ /dpts?:(\S+)/) {
-                        $d_port = $1;
+                if ($rule{'extended'}) {
+                    if ($rule{'protocol'} eq 'tcp'
+                            or $rule{'protocol'} eq 'udp') {
+                        my $s_port  = '0:0';  ### any to any
+                        my $d_port  = '0:0';
+                        if ($rule{'extended'} =~ /dpts?:(\S+)/) {
+                            $d_port = $1;
+                        }
+                        if ($rule{'extended'} =~ /spts?:(\S+)/) {
+                            $s_port = $1;
+                        }
+                        $rule{'s_port'} = $rule{'sport'} = $s_port;
+                        $rule{'d_port'} = $rule{'dport'} = $d_port;
+
+                        for my $state_hr (@global_accept_state) {
+                            next unless $state_hr->{'src'} eq '0.0.0.0/0';
+                            next unless $state_hr->{'dst'} eq '0.0.0.0/0';
+                            next unless $state_hr->{'proto'} eq 'all' or
+                                $state_hr->{'proto'} = $rule{'proto'};
+                            next unless $state_hr->{'intf_in'} eq '*' or
+                                $state_hr->{'intf_in'} eq $rule{'intf_in'};
+                            next unless $state_hr->{'intf_out'} eq '*' or
+                                $state_hr->{'intf_out'} eq $rule{'intf_out'};
+                            ### if we make it here, then the state rule
+                            ### applies to this rule
+                            $rule{'state'} = $state_hr->{'state'};
+                        }
                     }
-                    if ($rule{'extended'} =~ /spts?:(\S+)/) {
-                        $s_port = $1;
+                    if ($rule{'target'} eq 'ACCEPT'
+                            and $rule{'extended'} =~ m|^state\s+(\S+)|) {
+                        my $state_str = $1;
+                        if ($state_str =~ /ESTABLISHED/
+                                or $state_str =~ /RELATED/) {
+
+                            push @global_accept_state, {
+                                'state'    => $state_str,
+                                'src'      => $rule{'src'},
+                                'dst'      => $rule{'dst'},
+                                'intf_in'  => $rule{'intf_in'},
+                                'intf_out' => $rule{'intf_out'},
+                                'proto'    => $rule{'protocol'}
+                            };
+                            my %state_hash = ();
+                        }
                     }
-                    $rule{'s_port'} = $rule{'sport'} = $s_port;
-                    $rule{'d_port'} = $rule{'dport'} = $d_port;
                 }
-            } else {
-                $rule{'raw'} = $line;
             }
         } else {
             ### ACCEPT tcp  -- 164.109.8.0/24  0.0.0.0/0  tcp dpt:22 flags:0x16/0x02
@@ -198,7 +230,9 @@ sub chain_rules() {
 
             if ($line =~ m|^\s*(\S+)\s+(\S+)\s+\-\-\s+(\S+)\s+(\S+)\s*(.*)|) {
                 $rule{'target'}   = $1;
-                $rule{'protocol'} = $rule{'proto'} = $2;
+                my $proto = $2;
+                $proto = 'all' if $proto eq '0';
+                $rule{'protocol'} = $rule{'proto'} = $proto;
                 $rule{'src'}      = $3;
                 $rule{'dst'}      = $4;
                 $rule{'extended'} = $5;
@@ -216,8 +250,6 @@ sub chain_rules() {
                     $rule{'s_port'} = $rule{'sport'} = $s_port;
                     $rule{'d_port'} = $rule{'dport'} = $d_port;
                 }
-            } else {
-                $rule{'raw'} = $line;
             }
         }
         push @chain, \%rule;
@@ -279,6 +311,10 @@ sub default_drop() {
             my $proto  = $1;
             my $p_tmp  = $2;
             my $prefix = 'NONE';
+
+            ### some recent iptables versions return "0" instead of "all"
+            ### for the protocol number
+            $proto = 'all' if $proto eq '0';
             ### LOG flags 0 level 4 prefix `DROP '
             if ($p_tmp && $p_tmp =~ m|LOG.*\s+prefix\s+
                 \`\s*(.+?)\s*\'|x) {
@@ -289,6 +325,8 @@ sub default_drop() {
             $protocols{$proto}{'LOG'}{'rulenum'} = $rule_ctr;
         } elsif ($policy eq 'ACCEPT' and $line =~ m|^DROP\s+(\w+)\s+\-\-\s+.*
             $any_ip_re\s+$any_ip_re\s*$|x) {
+            my $proto = $1;
+            $proto = 'all' if $proto eq '0';
             ### DROP    all  --  0.0.0.0/0     0.0.0.0/0
             $protocols{$1}{'DROP'} = $rule_ctr;
         }
@@ -333,11 +371,22 @@ sub default_log() {
         };
     }
 
+    ### determine the output style (e.g. "-nL -v" or just plain "-nL"; if the
+    ### policy data came from a file then -v might not have been used)
+    my $ipt_verbose = 0;
+    for my $line (@ipt_lines) {
+        if ($line =~ /^\s*pkts\s+bytes\s+target/) {
+            $ipt_verbose = 1;
+            last;
+        }
+    }
+
     return '[-] Could not get iptables output!', 0
         unless @ipt_lines;
 
     ### first get all logging rules and associated chains
     my $log_chain;
+
     for my $line (@ipt_lines) {
         chomp $line;
 
@@ -350,14 +399,28 @@ sub default_log() {
         $log_chain = '' unless $line =~ /\S/;
         next unless $log_chain;
 
-        if ($line =~ m|^\s*U?LOG\s+(\w+)\s+\-\-\s+.*$any_ip_re
-                \s+$any_ip_re\s+.*U?LOG|x) {
-            ### the above regex allows the limit target to be used
-            $log_chains{$log_chain}{$1} = '';  ### protocol
-
-            if ($log_chain eq $chain) {
-                $log_rules{$1} = '';
+        my $proto = '';
+        my $found = 0;
+        if ($ipt_verbose) {
+            if ($line =~ m|^\s*\d+\s+\d+\s*U?LOG\s+(\w+)\s+\-\-\s+
+                    \S+\s+\S+\s+$any_ip_re
+                    \s+$any_ip_re\s+.*U?LOG|x) {
+                $proto = $1;
+                $found = 1;
             }
+        } else {
+            if ($line =~ m|^\s*U?LOG\s+(\w+)\s+\-\-\s+$any_ip_re
+                    \s+$any_ip_re\s+.*U?LOG|x) {
+                $proto = $1;
+                $found = 1;
+            }
+        }
+
+        if ($found) {
+            $proto = 'all' if $proto eq '0';
+            ### the above regex allows the limit target to be used
+            $log_chains{$log_chain}{$proto} = '';  ### protocol
+            $log_rules{$proto} = '' if $log_chain eq $chain;
         }
     }
 
