@@ -4,7 +4,7 @@
 *  File: psadwatchd.c
 *
 *  Purpose: psadwatchd checks on an interval of every five seconds to make
-*           sure that both kmsgsd and psad are running on the box.  If
+*           sure that both kmsgsd and psad are running on the box. If
 *           either daemon has died, psadwatchd will restart it and notify
 *           each email address in @email_addresses that the daemon has been
 *           restarted.
@@ -13,32 +13,30 @@
 *
 *  Credits:  (see the CREDITS file)
 *
-*  Copyright (C) 1999-2006 Michael Rash (mbr@cipherdyne.org)
+*  Copyright (C) 1999-2007 Michael Rash (mbr@cipherdyne.org)
 *
 *  License (GNU Public License):
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
-*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
-*     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+*     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *     USA
 *
 *****************************************************************************
 *
-*  $Id: psadwatchd.c 1947 2007-01-27 06:44:00Z mbr $
+*  $Id: psadwatchd.c 2047 2007-05-23 04:15:31Z mbr $
 */
 
 /* includes */
 #include "psad.h"
 
 /* defines */
-#define PSADWATCHD_CONF "/etc/psad/psadwatchd.conf"
-#define PSAD_CONF "/etc/psad/psad.conf" /* only used for DATA_INPUT_METHOD */
-#define ALERT_CONF "/etc/psad/alert.conf"
+#define CONFIG_FILE "/etc/psad/psad.conf" /* only used for DATA_INPUT_METHOD */
 
 /* globals */
 short int psad_syscalls_ctr     = 0;
@@ -51,7 +49,6 @@ char mail_addrs[MAX_EMAIL_LEN];
 char shCmd[MAX_GEN_LEN];
 char mailCmd[MAX_GEN_LEN];
 char config_file[MAX_PATH_LEN];
-char alert_config_file[MAX_PATH_LEN];
 char alerting_methods[MAX_GEN_LEN];
 char psadCmd[MAX_PATH_LEN];
 char psad_pid_file[MAX_PATH_LEN];
@@ -68,7 +65,6 @@ static volatile sig_atomic_t received_sighup = 0;
 
 /* prototypes */
 static void parse_config(void);
-static void parse_alert_config(void);
 static void expand_config_vars(void);
 static void find_sub_var_value(
     char *value,
@@ -83,7 +79,6 @@ static void check_process(
     const char *binary_path,
     unsigned int max_retries
 );
-static void check_data_input_mode(void);
 static void incr_syscall_ctr(const char *pid_name, unsigned int max_retries);
 static void reset_syscall_ctr(const char *pid_name);
 static void give_up(const char *pid_name);
@@ -102,20 +97,15 @@ int main(int argc, char *argv[]) {
     sleep(1);
 #endif
 
-    strlcpy(config_file, PSADWATCHD_CONF, MAX_PATH_LEN);
-    strlcpy(alert_config_file, ALERT_CONF, MAX_PATH_LEN);
+    strlcpy(config_file, CONFIG_FILE, MAX_PATH_LEN);
 
-    while((cmdlopt = getopt(argc, argv, "c:a:")) != -1) {
+    while((cmdlopt = getopt(argc, argv, "c:")) != -1) {
         switch(cmdlopt) {
             case 'c':
                 strlcpy(config_file, optarg, MAX_PATH_LEN);
                 break;
-            case 'a':
-                strlcpy(alert_config_file, optarg, MAX_PATH_LEN);
-                break;
             default:
-                printf("[+] Usage: psadwatchd [-c <config file>] ");
-                printf("[-a <alert config file>]\n");
+                printf("[+] Usage: psadwatchd [-c <config file>]\n");
                 exit(EXIT_FAILURE);
         }
     }
@@ -126,16 +116,14 @@ int main(int argc, char *argv[]) {
 
     /* parse the config files */
     parse_config();
-    parse_alert_config();
 
 #ifdef DEBUG
     dump_config();
 #endif
 
     /* see if we are suppose to disable all email alerts */
-    if (strncmp("noemail", alerting_methods, MAX_GEN_LEN) == 0) {
+    if (strncmp("noemail", alerting_methods, MAX_GEN_LEN) == 0)
         no_email = 1;
-    }
 
     /* first make sure there isn't another psadwatchd already running */
     check_unique_pid(psadwatchd_pid_file, "psadwatchd");
@@ -173,7 +161,6 @@ int main(int argc, char *argv[]) {
             /* reparse the config file since we received a
              * HUP signal */
             parse_config();
-            parse_alert_config();
 
             slogr("psad(psadwatchd)",
                     "received HUP signal, re-imported psadwatchd.conf");
@@ -276,7 +263,7 @@ static void incr_syscall_ctr(const char *pid_name, unsigned int max_retries)
         psad_syscalls_ctr++;
 #ifdef DEBUG
         fprintf(stderr,
-            "[-] %s not running.  Trying to restart (%d tries so far).\n",
+            "[-] %s not running. Trying to restart (%d tries so far).\n",
             pid_name, psad_syscalls_ctr);
 #endif
         if (psad_syscalls_ctr >= max_retries)
@@ -285,7 +272,7 @@ static void incr_syscall_ctr(const char *pid_name, unsigned int max_retries)
         kmsgsd_syscalls_ctr++;
 #ifdef DEBUG
         fprintf(stderr,
-            "[-] %s not running.  Trying to restart (%d tries so far).\n",
+            "[-] %s not running. Trying to restart (%d tries so far).\n",
             pid_name, kmsgsd_syscalls_ctr);
 #endif
         if (kmsgsd_syscalls_ctr >= max_retries)
@@ -308,13 +295,13 @@ static void give_up(const char *pid_name)
 {
     char mail_str[MAX_MSG_LEN] = "";
 #ifdef DEBUG
-    fprintf(stderr, "[*] Could not restart %s process.  Exiting.\n", pid_name);
+    fprintf(stderr, "[*] Could not restart %s process. Exiting.\n", pid_name);
 #endif
     strlcat(mail_str, " -s \"[*] psadwatchd: Could not restart ", MAX_MSG_LEN);
     strlcat(mail_str, pid_name, MAX_MSG_LEN);
     strlcat(mail_str, " on ", MAX_MSG_LEN);
     strlcat(mail_str, hostname, MAX_MSG_LEN);
-    strlcat(mail_str, ".  Exiting.\" ", MAX_MSG_LEN);
+    strlcat(mail_str, ". Exiting.\" ", MAX_MSG_LEN);
     strlcat(mail_str, mail_addrs, MAX_MSG_LEN);
     strlcat(mail_str, mail_redr, MAX_MSG_LEN);
 
@@ -334,10 +321,8 @@ static void exec_binary(const char *binary, const char *cmdlinefile)
     pid_t child_pid;
     int arg_num=0, non_ws, i;
 
-    prog_argv[arg_num] = (char *) malloc(strlen(binary)+1);
-    if (prog_argv[arg_num] == NULL) {
-        exit(EXIT_FAILURE);
-    }
+    prog_argv[arg_num] = (char *) safe_malloc(strlen(binary)+1);
+
     strlcpy(prog_argv[arg_num], binary, strlen(binary)+1);
     arg_num++;
 
@@ -369,10 +354,8 @@ static void exec_binary(const char *binary, const char *cmdlinefile)
             if (arg_num >= MAX_ARG_LEN)
                 exit(EXIT_FAILURE);
 
-            prog_argv[arg_num] = (char *) malloc(non_ws+1);
-            if (prog_argv[arg_num] == NULL) {
-                exit(EXIT_FAILURE);
-            }
+            prog_argv[arg_num] = (char *) safe_malloc(non_ws+1);
+
             for (i=0; i<non_ws; i++)
                 prog_argv[arg_num][i] = *(index - (non_ws - i));
             prog_argv[arg_num][i] = '\0';
@@ -386,10 +369,6 @@ static void exec_binary(const char *binary, const char *cmdlinefile)
 
     if (arg_num >= MAX_ARG_LEN)
         exit(EXIT_FAILURE);
-    prog_argv[arg_num] = (char *) malloc(1);
-    if (prog_argv[arg_num] == NULL) {
-        exit(EXIT_FAILURE);
-    }
     prog_argv[arg_num] = NULL;
 
     if ((child_pid = fork()) < 0)
@@ -414,10 +393,10 @@ static void parse_config(void)
     FILE *config_ptr;         /* FILE pointer to the config file */
     int linectr = 0;
     char config_buf[MAX_LINE_BUF];
+    char data_input_mode[MAX_GEN_LEN];
     char *index;
 
-    /* check to see if kmsgsd needs to be running */
-    check_data_input_mode();  /* note that this also gets EMAIL_ADDRESSES */
+    data_input_mode[0] = '\0';
 
     if ((config_ptr = fopen(config_file, "r")) == NULL) {
         perror("[*] Could not open config file");
@@ -442,12 +421,15 @@ static void parse_config(void)
             find_char_var("PSAD_RUN_DIR", psad_run_dir, index);
             find_char_var("PSAD_PID_FILE", psad_pid_file, index);
             find_char_var("PSAD_CMDLINE_FILE", psad_cmdline_file, index);
+            find_char_var("ALERTING_METHODS", alerting_methods, index);
             find_char_var("KMSGSD_PID_FILE", kmsgsd_pid_file, index);
             find_char_var("PSADWATCHD_PID_FILE", psadwatchd_pid_file, index);
             find_char_var("PSADWATCHD_CHECK_INTERVAL",
                 char_psadwatchd_check_interval, index);
             find_char_var("PSADWATCHD_MAX_RETRIES",
                 char_psadwatchd_max_retries, index);
+            find_char_var("SYSLOG_DAEMON", data_input_mode, index);
+            find_char_var("EMAIL_ADDRESSES", mail_addrs, index);
 
             /* commands */
             find_char_var("kmsgsdCmd", kmsgsdCmd, index);
@@ -458,6 +440,10 @@ static void parse_config(void)
     }
     fclose(config_ptr);
 
+    /* see if we are using the ulogd mode */
+    if (strncmp(data_input_mode, "ulogd", MAX_GEN_LEN) == 0)
+        check_kmsgsd = 0;
+
     /* resolve any embedded variables */
     expand_config_vars();
 
@@ -550,7 +536,7 @@ static void expand_config_vars(void)
             found_sub_var = 1;
         }
 
-        if (has_sub_var("ksmgsdCmd", kmsgsdCmd, sub_var,
+        if (has_sub_var("kmsgsdCmd", kmsgsdCmd, sub_var,
                 pre_str, post_str)) {
             find_sub_var_value(kmsgsdCmd, sub_var, pre_str, post_str);
             found_sub_var = 1;
@@ -624,81 +610,6 @@ static void find_sub_var_value(char *value, char *sub_var, char *pre_str,
     return;
 }
 
-static void check_data_input_mode(void)
-{
-    FILE *config_ptr;   /* FILE pointer to the config file */
-    char config_buf[MAX_LINE_BUF];
-    char data_input_mode[MAX_GEN_LEN];
-    char *index;
-
-    if ((config_ptr = fopen(PSAD_CONF, "r")) == NULL) {
-        fprintf(stderr, "[-] Could not open %s for reading.\n",
-            PSAD_CONF);
-        exit(EXIT_FAILURE);
-    }
-
-    data_input_mode[0] = '\0';
-
-    /* increment through each line of the config file */
-    while ((fgets(config_buf, MAX_LINE_BUF, config_ptr)) != NULL) {
-        /* set the index pointer to the beginning of the line */
-        index = config_buf;
-
-        /* advance the index pointer through any whitespace
-         * at the beginning of the line */
-        while (*index == ' ' || *index == '\t') index++;
-
-        /* skip comments and blank lines, etc. */
-        if ((*index != '#') && (*index != '\n') &&
-                (*index != ';') && (index != NULL)) {
-
-            find_char_var("SYSLOG_DAEMON", data_input_mode, index);
-            find_char_var("EMAIL_ADDRESSES", mail_addrs, index);
-        }
-    }
-    fclose(config_ptr);
-
-    /* see if we are using the ulogd mode */
-    if (strncmp(data_input_mode, "ulogd", MAX_GEN_LEN) == 0)
-        check_kmsgsd = 0;
-
-    return;
-}
-
-static void parse_alert_config(void)
-{
-    FILE *config_ptr;         /* FILE pointer to the config file */
-    int linectr = 0;
-    char config_buf[MAX_LINE_BUF];
-    char *index;
-
-    if ((config_ptr = fopen(alert_config_file, "r")) == NULL) {
-        perror("[*] Could not open config file");
-        exit(EXIT_FAILURE);
-    }
-
-    /* increment through each line of the config file */
-    while ((fgets(config_buf, MAX_LINE_BUF, config_ptr)) != NULL) {
-        linectr++;
-        index = config_buf;  /* set the index pointer to the
-                                beginning of the line */
-
-        /* advance the index pointer through any whitespace
-         * at the beginning of the line */
-        while (*index == ' ' || *index == '\t') index++;
-
-        /* skip comments and blank lines, etc. */
-        if ((*index != '#') && (*index != '\n') &&
-                (*index != ';') && (index != NULL)) {
-
-            find_char_var("ALERTING_METHODS", alerting_methods, index);
-        }
-    }
-    fclose(config_ptr);
-
-    return;
-}
-
 #ifdef DEBUG
 static void dump_config(void)
 {
@@ -709,6 +620,7 @@ static void dump_config(void)
     fprintf(stderr, "    PSAD_PID_FILE: %s\n", psad_pid_file);
     fprintf(stderr, "    PSAD_CMDLINE_FILE: %s\n", psad_cmdline_file);
     fprintf(stderr, "    KMSGSD_PID_FILE: %s\n", kmsgsd_pid_file);
+    fprintf(stderr, "    ALERTING_METHODS: %s\n", alerting_methods);
     fprintf(stderr, "    PSADWATCHD_PID_FILE: %s\n", psadwatchd_pid_file);
     fprintf(stderr, "    PSADWATCHD_CHECK_INTERVAL: %u\n",
         psadwatchd_check_interval);
